/*************************************************************************
GENERAL Lockfile header file
--------------------------------------------------------------------------
Copyright (c) 1992 Anthony Rumble
--------------------------------------------------------------------------
RCS Info

$Header: /home/smilie/emsi/RCS/lockfile.h,v 1.2 1992/12/06 09:35:50 smilie Exp $

$Log: lockfile.h,v $
 * Revision 1.2  1992/12/06  09:35:50  smilie
 * forgot extern rmlock
 *
 * Revision 1.1  1992/10/09  13:19:00  smilie
 * Initial revision
 *

$Id: lockfile.h,v 1.2 1992/12/06 09:35:50 smilie Exp $

*************************************************************************/
#ifndef _LOCKFILE_H
#define _LOCKFILE_H

#include <sys/types.h>

extern char lock[256];			/* lock file name */

extern void rmlocks(void);		/* remove lock file */
extern pid_t readlock(char *);		/* read a lock file */
extern int checklock(char *);		/* check a lock file */
extern int makelock(char *);		/* make a lock file */
extern void rmlock(char *);		/* remove a specific lock file */

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#endif

